from importlib import metadata as importlib_metadata

from . import path
from . import gmat
from . import gene
from . import gcell
from . import locus
from . import sc

# __all__ = [
#     "gene",
#     "gcell",
#     "locus",
#     "sc",
# ]